/* -*- C++ -*-
 * Serene Programming Language
 *
 * Copyright (c) 2019-2023 Sameer Rahmani <lxsameer@gnu.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "serene/exprs/expression.h"
#include "serene/exprs/list.h"
#include "serene/exprs/symbol.h"

#include "../test_helpers.cpp.inc"

#include <llvm/ADT/ArrayRef.h>

namespace serene {
namespace exprs {

TEST_CASE("Public Expression API", "[expression]") {
  std::unique_ptr<reader::LocationRange> range(dummyLocation());
  auto sym = make<Symbol>(*range, "example", "user");

  REQUIRE(sym->getType() == ExprType::Symbol);
  CHECK(sym->toString() == "<Symbol user/example>");

  auto list = makeAndCast<List>(*range, sym);

  CHECK(list->toString() == "<List  <Symbol user/example>>");
};

} // namespace exprs
} // namespace serene
